package edu.umd.dawn.common.wrappers;

import edu.umd.dawn.common.jwt.Claims;
import edu.umd.dawn.common.jwt.JWTUtil;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletRequestWrapper;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class DawnHttpServletRequestWrapper extends HttpServletRequestWrapper {

    private String accessSecret;

    public DawnHttpServletRequestWrapper(HttpServletRequest request) {
        super(request);
    }

    public DawnHttpServletRequestWrapper(HttpServletRequest request, String accessSecret) {
        super(request);
        this.accessSecret = accessSecret;
    }

    /**
     * try everything possible to get the claims
     * eventually, we will be requiring jwts everywhere so this wont need to be so complex
     * @return
     */
    public Claims getClaims() {
        Claims claims = (Claims) getAttribute("claims");
        if (claims != null) {
            return claims;
        }

        if (accessSecret.equals("empty")) {
            log.warn("This service has not been provided the configuration value for `config.accessSecret` in the"
                    + " application.properties configuration. Since it uses the JWTInterceptor, this is a required"
                    + " value for proper functionality.");
        }

        String token = (String) getAttribute("token");
        if (token != null) {
            try {
                return JWTUtil.parse(accessSecret, token).getClaims();
            } catch (Exception e) {
            }
        }

        String jwtHeader = (String) getHeader("Authorization");
        if (jwtHeader == null) {
            return null;
        }
        if (jwtHeader.startsWith("Bearer ")) {
            jwtHeader.replace("Bearer ", "");
        }

        try {
            return JWTUtil.parse(accessSecret, jwtHeader).getClaims();
        } catch (Exception e) {
        }

        return null;
    }
}
